// Persistence of Vision Ray Tracer Scene Description File
// File: AK_My Favourite Isosurface.pov
// Vers: 3.8
// Desc: Isosurface code for a complex ring structure
// Date: 20 November 2003
// Auth: Alex Kluchikov, klk### [at] ukrnet, akl### [at] mailru
//       amended by Tor Olav Kristensen
// in:   povray.text.scene-files

//+w640 +h640 +a0.3 +am2 +bm2 +bs8 +wt6

#version 3.8;

global_settings {
  assumed_gamma 1.0
}

#include "functions.inc"  // For f_r() and f_sphere()

//=================================================
#declare NakedIso = off;
//=================================================

#declare TwoPi = 2*pi;
#declare TT = 8; // Number of "Turns" [8]
#declare M1 = TT/3;
#declare M2 = 5; // Number of "Blobs" [5]

#declare Rmaj = 1.50;
#declare Rmin = 0.25;

#declare dAngle = TwoPi/3;

#declare a0 = Rmin*cos(0*dAngle);
#declare b0 = Rmin*sin(0*dAngle);

#declare a1 = Rmin*cos(1*dAngle);
#declare b1 = Rmin*sin(1*dAngle);

#declare a2 = Rmin*cos(2*dAngle);
#declare b2 = Rmin*sin(2*dAngle);

#declare TempFn0 =
  function(A, B) {
    pow(f_r(A + a0, 0, B + b0), 1/32)
   +pow(f_r(A + a1, 0, B + b1), 1/32)
   +pow(f_r(A + a2, 0, B + b2), 1/32)
  }

#declare TempFn1 =
  function(XZ, YY, Cos, Sin, RF) {
    0.330*TempFn0(+XZ*Cos +YY*Sin, -XZ*Sin +YY*Cos)
   +0.010*sin(RF)
   -0.945
  }

#declare TempFn2 =
  function(XZ, YY, Phi) {
    TempFn1(XZ, YY, cos(Phi*M1), sin(Phi*M1), Phi*M2)
  }

#if (NakedIso)
isosurface {
  function { TempFn2(f_sphere(x, 0, z, Rmaj), y, 90 - atan2(x, z)) }
  threshold 0.0  //[0.0]
  accuracy 2E-4
  max_gradient 0.75 //[0.75]
  contained_by {
    box {
     -<Rmaj + Rmin, Rmin, Rmaj + Rmin> - <1, 1, 1>*0.25,  //[*0.12]
      <Rmaj + Rmin, Rmin, Rmaj + Rmin> + <1, 1, 1>*0.25   //[*0.12]
    }
  }
  scale 1.2
  rotate -90*x
  pigment {srgb 1}
}

#else

#declare Heart =
isosurface {
  function { TempFn2(f_sphere(x, 0, z, Rmaj), y, 90 - atan2(x, z)) }
  threshold 0.0  //[0.0]
  accuracy 2E-4
  max_gradient 1.3 //[0.75]
  contained_by {
    box {
     -<Rmaj + Rmin, Rmin, Rmaj + Rmin> - <1, 1, 1>*0.25,  //[*0.12]
      <Rmaj + Rmin, Rmin, Rmaj + Rmin> + <1, 1, 1>*0.25   //[*0.12]
    }
  }
}

object {Heart
  scale 1.2
  rotate -90*x
  pigment {srgb 1}
}

#end

// ===== 1 ======= 2 ======= 3 ======= 4 ======= 5 ======= 6 ======= 7

sky_sphere {
  pigment {
    gradient y 
    color_map {
      [0, color rgb <0.8,0.83,1>]
      [1, color rgb <0.4,0.7,1>]
    }
    scale 2 
    translate -y
  }
}

sphere {0, 1000
  texture {
    pigment {
      average 
      pigment_map {
        #declare gr=.8;
        #declare ge=3/(1+gr+gr);
        #declare t1=.3;
        #declare t2=.8;
        #declare tr1=.1;
        #declare tr2=.1;
        [1, bozo translate 5*y 
            color_map {
              [t1, color rgbt < 1,gr,gr,tr1>*ge]
              [t2, color rgbt < 0, 0, 0,tr2>*ge]
            }]
        [1, bozo translate 5*z 
            color_map {
              [t1, color rgbt < gr, 1,gr,tr1>*ge]
              [t2, color rgbt <  0, 0, 0,tr2>*ge]
            }]
        [1, bozo translate 5*x 
            color_map {
              [t1, color rgbt < gr,gr, 1,tr1>*ge]
              [t2, color rgbt <  0, 0, 0,tr2>*ge]
            }]
      }
    }
    scale 0.25
    finish {emission 1 diffuse 0}
    scale 3000
  }
  no_shadow
  no_image
  hollow
  //no_reflection
  double_illuminate
  scale 100
}

#declare area = 0;
#declare areacount = 2;
#declare areasize = 2.5;

light_source {
  -35*z, color rgb <1.0,0.8,0.6>*0.55
  #if (area)
    area_light x*areasize,y*areasize,areacount,areacount jitter adaptive 1
    circular orient
  #else
    parallel
  #end
  rotate x*55 rotate -y*45
}

light_source {
  -35*z, color rgb <0.6,0.8,1>*0.55
  #if (area)
    area_light x*areasize,y*areasize,areacount,areacount jitter adaptive 1
    circular orient
  #else
    parallel
  #end
  rotate x*25 rotate y*25
}

camera {
  location -5*z
  look_at 0
  right x*image_width/image_height
}

